use crate::jira_config::JiraConfig;
use reqwest::header::{HeaderMap, HeaderValue, CONTENT_TYPE};
use serde::{Deserialize, Serialize};

#[derive(Debug, Deserialize, Serialize)]
struct JiraIssue {
    key: String,
    #[serde(rename = "self")]
    href: String,
    fields: JiraIssueResponseFields,
}

#[derive(Debug, Deserialize, Serialize)]
struct JiraIssueResponseFields {
    summary: String,
    status: Status,
    created: chrono::DateTime<chrono::Utc>,
    priority: Priority,
    assignee: Person,
    reporter: Person,
    creator: Person,
    #[serde(rename = "duedate")]
    due_date: Option<chrono::NaiveDate>,
}

#[derive(Deserialize)]
struct JiraSearchResponse {
    issues: Vec<JiraIssue>,
    total: u32,
}

#[derive(Debug, Deserialize, Serialize)]
struct Status {
    name: String,
}

#[derive(Debug, Deserialize, Serialize)]
struct Priority {
    name: String,
    id: String,
}

#[derive(Debug, Deserialize, Serialize)]
struct Person {
    #[serde(rename = "self")]
    href: String,
    #[serde(rename = "displayName")]
    display_name: String,
    #[serde(rename = "accountId")]
    account_id: String,
    #[serde(rename = "emailAddress")]
    email_address: String,
}

async fn list_jira_issues(
    config: &JiraConfig,
) -> Result<JiraSearchResponse, Box<dyn std::error::Error>> {
    let client = reqwest::Client::new();

    let mut headers = HeaderMap::new();
    headers.insert(CONTENT_TYPE, HeaderValue::from_static("application/json"));

    // JQL query to find issues assigned to the specified user
    let jql =
        "assignee = currentUser() AND resolution = Unresolved order by updated DESC".to_string();
    let query = [("jql", jql.as_str())];

    let response = client
        .get(format!("{}/rest/api/2/search", config.url))
        .basic_auth(&config.email, Some(&config.api_token))
        .headers(headers)
        .query(&query)
        .send()
        .await?;

    if !response.status().is_success() {
        let error_text = response.text().await?;
        return Err(format!("Failed to fetch issues: {}", error_text).into());
    }

    Ok(response.json::<JiraSearchResponse>().await?)
}

fn display_issues_pretty(issues: &[JiraIssue], jira_url: &str) {
    println!("Found {} issues:", issues.len());
    println!("{:-<80}", "");

    for issue in issues {
        println!("Key:     {}", issue.key);
        println!("Summary: {}", issue.fields.summary);
        println!("Status:  {}", issue.fields.status.name);
        println!("Created: {}", issue.fields.created);
        println!("URL:     {}/browse/{}", jira_url, issue.key);
        println!("{:-<80}", "");
    }
}

fn display_issues_json(issues: &[JiraIssue]) -> Result<(), Box<dyn std::error::Error>> {
    let j = serde_json::to_string_pretty(issues)?;
    println!("{}", j);
    Ok(())
}

pub async fn list(json: bool) -> Result<(), Box<dyn std::error::Error>> {
    let config = JiraConfig::load().map_err(|e| format!("Configuration error: {}", e))?;
    if !json {
        println!("Fetching issues assigned...");
    }

    match list_jira_issues(&config).await {
        Ok(response) => {
            if json {
                if response.issues.is_empty() {
                    println!("[]");
                } else {
                    display_issues_json(&response.issues)?;
                }
            } else if response.issues.is_empty() {
                println!("No open issues found for assigned to you");
            } else {
                display_issues_pretty(&response.issues, &config.url);
                println!("Total issues: {}", response.total);
            }
        }
        Err(e) => {
            eprintln!("Error fetching issues: {}", e);
            std::process::exit(1);
        }
    }

    Ok(())
}
